@@ -1,5 +1,6 @@
 import sys
 import os
+import logging
 import csv
 from .azure_cis_tool import azure_cis as azure_cis_tool
 
@@ -24,12 +25,15 @@ def get_issues_from_csv_file(config_issues_csv, assetid):
     return findings
 
 def run_azure_cis_bench(args):
+    if args.assetid.strip() == "":
+        logging.error("[assetid] cannot be empty")
+        sys.exit(1)
     config_issues_csv = azure_cis_tool.run_tests()
     if config_issues_csv is None:
         sys.exit(1)
     asset = { }
     asset['id'] = args.assetid
-    asset['name'] = args.assetname if args.assetname else asset['id']
+    asset['name'] = args.assetname if args.assetname and args.assetname.strip() != "" else asset['id']
     asset['type'] = 'Azure'
     asset['owner'] = args.handle
     asset['products'] = []
