@@ -1,6 +1,9 @@
 import sys
 import platform
 import os
+import tempfile
+import shutil
+import uuid
 import subprocess
 import logging
 import socket
@@ -20,6 +23,7 @@ with warnings.catch_warnings():
    from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
    from cryptography.fernet import Fernet
 from . import utils
+from . import plugin_processor
 
 def check_host_up(host):
     if not host['remote']:
@@ -161,6 +165,21 @@ def discover_rh(args, host):
     logging.info("Completed retrieval of product details")
     return plist
 
+def discover_suse(args, host):
+    plist = []
+    cmdarr = ["/bin/rpm -qa"]
+    logging.info("Retrieving product details")
+    rpmout = utils.run_cmd_on_host(args, host, cmdarr)
+
+    for l in rpmout.splitlines():
+        l = l.strip()
+        l_tokens = l.split('-')
+        tokens_len = len(l_tokens)
+        pkg = "-".join(l_tokens[:-2])
+        plist.append(pkg+' '+l_tokens[-2]+'-'+l_tokens[-1])
+    logging.info("Completed retrieval of product details")
+    return plist
+
 def discover_ubuntu(args, host):
     plist = []
     cmdarr = ["/usr/bin/apt list --installed"]
@@ -205,7 +224,7 @@ def discover(args):
                             password = getpass.getpass(prompt="Enter password: ")
                             password = password.encode()
                     else:
-                        password = args.password
+                        password = args.password.encode()
                     salt = base64.b64encode(password)
                     kdf = PBKDF2HMAC(
                             algorithm=hashes.SHA256(),
@@ -305,24 +324,38 @@ def discover(args):
                         logging.error("Invalid password")
                         logging.error("Please use the same password as was used previously to secure the file")
                         return None
-            # secure the new rows in the file
-            with open(host_list_file, mode='w') as csvfile:
-                fieldnames = ['hostname','userlogin','userpwd','privatekey','assetname']
-                writer = csv.DictWriter(csvfile, fieldnames=fieldnames, quoting=csv.QUOTE_NONE, escapechar='\\')
-                writer.writeheader()
-                for h in remote_hosts:
-                    if h['userpwd'] != '' and not h['userpwd'].startswith('__SECURE__:'):
-                        h['userpwd'] = '__SECURE__:'+f.encrypt(h['userpwd'].encode('utf-8')).decode('utf-8')
-                    del h['remote']
-                    writer.writerow(h)
+            # create new file with secured information. This will be renamed subsequently
+            new_csv_file = uuid.uuid4().hex
+            new_csv_file = tempfile.gettempdir() + os.path.sep + new_csv_file + ".csv"
+            if os.path.isfile(new_csv_file):
+                os.remove(new_csv_file)
+            try:
+                # secure the new rows in the file
+                with open(new_csv_file, mode='w') as csvfile:
+                    fieldnames = ['hostname','userlogin','userpwd','privatekey','assetname']
+                    writer = csv.DictWriter(csvfile, fieldnames=fieldnames, quoting=csv.QUOTE_NONE, escapechar='\\')
+                    writer.writeheader()
+                    for h in remote_hosts:
+                        if h['userpwd'] != '' and not h['userpwd'].startswith('__SECURE__:'):
+                            h['userpwd'] = '__SECURE__:'+f.encrypt(h['userpwd'].encode('utf-8')).decode('utf-8')
+                        del h['remote']
+                        writer.writerow(h)
+            except Exception as err:
+                logging.error("Unable to save secured CSV file")
+                logging.error("%s", err)
+                os.remove(new_csv_file)
+                return None
+            os.remove(host_list_file)
+            shutil.copyfile(new_csv_file, host_list_file)
+            os.remove(new_csv_file)
             logging.info("Host list file secured")
             return None
         else:
             return discover_hosts(args, remote_hosts)
     else:
         host = { }
-        host['assetid'] = utils.get_ip() if args.assetid is None else args.assetid
-        host['assetname'] = host['assetid'] if args.assetname is None else args.assetname
+        host['assetid'] = utils.get_ip() if args.assetid is None or args.assetid.strip() == "" else args.assetid
+        host['assetname'] = host['assetid'] if args.assetname is None or args.assetname.strip() == "" else args.assetname
         host['hostname'] = utils.get_ip()
         host['remote'] = False
         hosts = [ host ]
@@ -393,7 +426,7 @@ def discover_host(args, host):
         asset_id = host['hostname']
         logging.warning("Using Hostname/IP [%s] for Asset [%s] as Identifier", asset_id, host['hostname'])
 
-    asset_name = host['assetname'] if host.get('assetname') is not None and len(host['assetname']) > 0 else asset_id
+    asset_name = host['assetname'] if host.get('assetname') is not None and len(host['assetname']) > 0 else host['hostname']
 
     asset_id = asset_id.replace(':','')
     asset_name = asset_name.replace('/','-')
@@ -412,6 +445,8 @@ def discover_host(args, host):
         plist = discover_macos(args, host)
     elif atype == "Alpine Linux":
         plist = discover_alpine(args, host)
+    elif atype == "Suse":
+        plist = discover_suse(args, host)
 
     if plist == None or len(plist) == 0:
         logging.error("Could not inventory asset [%s]", asset_id)
@@ -430,6 +465,8 @@ def discover_host(args, host):
     asset_tags.append(atype)
     asset_data['tags'] = asset_tags
 
+    plugin_processor.process_plugins(asset_data, host, args, '/')
+
     logging.info("Completed inventory discovery for asset [%s]", asset_id)
 
     return asset_data
@@ -511,7 +548,7 @@ def run_ssh_audit(args, assetid, ip):
             if algo not in key_issues:
                 key_issues[algo] = {}
                 key_issues[algo]['details'] = ''
-            reco = 'Recommentation: '+l.split('--')[1].strip()
+            reco = 'Recommendation: '+l.split('--')[1].strip()
             key_issues[algo]['details'] = key_issues[algo]['details'] + '\n'+ reco 
     for k in key_issues:
         if 'type' not in key_issues[k]:
