@@ -20,6 +20,7 @@ import re
 from . import utils as lib_utils
 from . import code_secrets as lib_code_secrets
 from . import sast
+from . import iac
 
 GIT_PATH = os.environ.get('GIT_PATH')
 if GIT_PATH is None:
@@ -28,7 +29,7 @@ if GIT_PATH is None:
     else:
         GIT_PATH = '/usr/bin/git'
 
-SUPPORTED_TYPES = ['pip', 'ruby', 'yarn', 'nuget', 'npm', 'maven', 'gradle', 'dll', 'jar', 'cargo']
+SUPPORTED_TYPES = ['pip', 'ruby', 'yarn', 'nuget', 'npm', 'maven', 'gradle', 'dll', 'jar', 'cargo', 'go']
 
 def cleanse_semver_version(pv):
     pv = pv.replace('"','')
@@ -49,12 +50,38 @@ def cleanse_semver_version(pv):
             pv = pv + " " + temp_tokens[token_index]
     return pv
 
+def discover_go_mod(args, localpath):
+    plist = []
+    files = lib_utils.find_files(localpath, 'go.mod')
+    prop_dict = None
+    for file_path in files:
+        fp = lib_utils.tw_open(file_path, args.encoding)
+        if fp == None:
+            continue
+        contents = fp.read()
+        fp.close()
+        if localpath.startswith('/tmp/'):
+            file_path = file_path.replace(localpath+'/','')
+        for line in contents.splitlines():
+            line = line.strip()
+            if line == '' or line.startswith('module ') or line.startswith('go ') or line.startswith('require (') or line.startswith(')') or line.startswith('replace') or line.startswith('exclude'):
+                continue
+            sp = line.split()
+            if len(sp) < 2:
+                continue
+            ver = sp[1]
+            if ver.startswith('v'):
+                ver = ver[1:]
+            prod = sp[0] + " " + ver + " source:"+file_path
+            plist.append(prod)
+    return plist, None
+
 def discover_cargo_toml(args, localpath):
     plist = []
     files = lib_utils.find_files(localpath, 'Cargo.toml')
     prop_dict = None
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -83,7 +110,7 @@ def discover_pom_xml(args, localpath):
     files = lib_utils.find_files(localpath, 'pom.xml')
     prop_dict = None
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -91,8 +118,8 @@ def discover_pom_xml(args, localpath):
         try:
             xmldoc = minidom.parseString(contents)
         except Exception:
-            logging.error("Unable to parse pom.xml")
-            return None
+            logging.error("Unable to parse pom.xml "+file_path)
+            continue
         if prop_dict is None:
             prop_dict = { }
             curr_prop_dict = prop_dict
@@ -163,7 +190,7 @@ def discover_gradle(args, localpath):
     plist = []
     files = lib_utils.find_files(localpath, 'dependencies.gradle')
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -181,11 +208,13 @@ def discover_gradle(args, localpath):
                     plist.append(pname)
     return plist, None
 
-def process_package_json_files(files, level, localpath):
+def process_package_json_files(files, args, localpath):
+    level = args.level
+    encoding = args.encoding
     plist = []
     p1list = [] # 1st level dependencies
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -223,7 +252,8 @@ def process_package_json_files(files, level, localpath):
                     for req_pname in req_dict:
                         if localpath.startswith('/tmp/'):
                             file_path = file_path.replace(localpath+'/','')
-                        pname = req_pname + ' ' + req_dict[req_pname] + " source:"+file_path
+                        pname = req_pname + ' ' + req_dict[req_pname]
+                        pname = cleanse_semver_version(pname) + " source:"+file_path
                         if pname not in plist:
                             plist.append(pname)
                 else:
@@ -256,20 +286,100 @@ def process_package_json_files(files, level, localpath):
                     p1list.append(pname)
     return plist, p1list 
 
+def filter_used_npm_dependencies(args, deplist, localpath):
+    logging.debug("Number of dependencies before used filter: "+str(len(deplist)))
+    dev_null_device = open(os.devnull, "w")
+    fdlist = []
+    f_handle, f_path = tempfile.mkstemp('.txt', 'tw-find-output-')
+    find_cmd = "find "+localpath+" -type f -name '*.js' -or -name '*.ts' > " + f_path
+    try:
+        out = subprocess.check_output([find_cmd], stderr=dev_null_device, shell=True)
+        out = out.decode(args.encoding)
+    except subprocess.CalledProcessError:
+        if logging_enabled:
+            logging.debug("Error running command...unable to filter used npm depdencies")
+        return deplist
+    for d in deplist:
+        dname = d.split()[0].strip()
+        if dname in fdlist:
+            continue
+        #print("Checking dependency for "+d)
+        cmd = "cat %s | xargs -r egrep -ni '(import|require|loader|plugins|%s).*['\"](%s|.?\d+)[\"']' -m 1 | wc -l " % (f_path, dname, dname)
+        #print(cmd)
+        try:
+            out = subprocess.check_output([cmd], stderr=dev_null_device, shell=True)
+            out = out.decode(args.encoding)
+        except subprocess.CalledProcessError:
+            if logging_enabled:
+                logging.debug("Error running command")
+            dev_null_device.close()
+            continue
+        if out.strip() != '0':
+            #print(dname+" is used")
+            fdlist.append(d)
+    dev_null_device.close()
+    os.remove(f_path)
+    logging.debug("Number of dependencies after used filter: "+str(len(fdlist)))
+    return fdlist
+
+def filter_used_dotnet_dependencies(args, deplist, localpath):
+    logging.debug("Number of dependencies before used filter: "+str(len(deplist)))
+    dev_null_device = open(os.devnull, "w")
+    fdlist = []
+    f_handle, f_path = tempfile.mkstemp('.txt', 'tw-find-output-')
+    find_cmd = "find "+localpath+" -type f -name '*.cs' > " + f_path
+    try:
+        out = subprocess.check_output([find_cmd], stderr=dev_null_device, shell=True)
+        out = out.decode(args.encoding)
+    except subprocess.CalledProcessError:
+        if logging_enabled:
+            logging.debug("Error running command...unable to filter used npm depdencies")
+        return deplist
+    for d in deplist:
+        dname = d.split()[0].strip()
+        if dname in fdlist:
+            continue
+        #logging.debug("Checking dependency for "+d)
+        cmd = "cat %s | xargs -r egrep -ni 'using.*%s' -m 1 | wc -l " % (f_path, dname)
+        try:
+            out = subprocess.check_output([cmd], stderr=dev_null_device, shell=True)
+            out = out.decode(args.encoding)
+        except subprocess.CalledProcessError:
+            if logging_enabled:
+                logging.debug("Error running command")
+            dev_null_device.close()
+            continue
+        if out.strip() != '0':
+            #logging.debug(dname+" is used")
+            fdlist.append(d)
+    dev_null_device.close()
+    os.remove(f_path)
+    logging.debug("Number of dependencies after used filter: "+str(len(fdlist)))
+    return fdlist
+
 def discover_package_json(args, localpath):
     files = lib_utils.find_files(localpath, 'package-lock.json')
     if len(files) > 0:
-        plist, p1list = process_package_json_files(files, args.level, localpath)
+        plist, p1list = process_package_json_files(files, args, localpath)
     else:
         files = lib_utils.find_files(localpath, 'package.json')
-        plist, p1list = process_package_json_files(files, args.level, localpath)
+        plist, p1list = process_package_json_files(files, args, localpath)
+    if len(plist) > 0:
+        if args.include_unused_dependencies == False:
+            logging.debug("Filtering out unused npm dependencies. This may take some time...")
+            plist = filter_used_npm_dependencies(args, plist, localpath)
+        else:
+            logging.warn("Including unused dependencies")
+            logging.warn("May increase false positives")
     return plist, p1list
 
 def discover_packages_config(args, localpath):
     plist = []
-    files = lib_utils.find_files(localpath, 'packages.config')
-    for file_path in files:
-        fp = open(file_path, 'r')
+    # Give first preference for dependencies from .csproj files
+    verprops = {}
+    propfiles = lib_utils.find_files(localpath, '.props')
+    for propfile in propfiles:
+        fp = open(propfile, mode='r')
         if fp == None:
             continue
         contents = fp.read()
@@ -277,8 +387,89 @@ def discover_packages_config(args, localpath):
         try:
             xmldoc = minidom.parseString(contents)
         except Exception:
-            logging.error("Unable to parse package config")
-            return None
+            traceback.print_exc()
+            logging.error("Unable to parse propfile file: %s", propfile)
+            continue
+        pglist = xmldoc.getElementsByTagName('PropertyGroup')
+        for pg in pglist:
+            props = pg.childNodes
+            for p in props:
+                if p.nodeType != p.TEXT_NODE: 
+                    if p.childNodes and len(p.childNodes) == 1 and p.tagName.endswith('Version'):
+                        verprops[p.tagName] = p.childNodes[0].data
+    files = lib_utils.find_files(localpath, '.csproj')
+    for file_path in files:
+        fp = open(file_path, mode='r')
+        if fp == None:
+            continue
+        contents = fp.read()
+        xmldoc = None
+        try:
+            xmldoc = minidom.parseString(contents)
+        except Exception:
+            traceback.print_exc()
+            logging.error("Unable to parse csproj file: %s", file_path)
+            continue
+        temp_plist = xmldoc.getElementsByTagName('PackageReference')
+        for p in temp_plist:
+            libname = p.getAttribute('Include')
+            if libname is None or len(libname.strip()) == 0:
+                continue
+            libver = p.getAttribute('Version')
+            if localpath.startswith('/tmp/'):
+                file_path = file_path.replace(localpath+'/','')
+            if libver is not None and len(libver.strip()) > 0 and not libver.startswith('$('):
+                if '*' in libver:
+                    libver = libver.replace('.*', '.0')
+                if ',' in libver:
+                    libver = libver.replace('(', '')
+                    libver = libver.replace(')', '')
+                    libver = libver.replace('[', '')
+                    libver = libver.replace(']', '')
+                    tokens = libver.split(',')
+                    if len(tokens[0].strip()) > 0:
+                        libver = tokens[0].strip()
+                    elif len(tokens[1].strip()) > 0:
+                        libver = tokens[1].strip()
+                    else:
+                        libver = '' # this case should never occur
+                pname = libname + ' ' + libver + " source:"+file_path
+            elif libver is not None and libver.startswith('$('):
+                verlabel = libver.replace('$(','').replace(')','')
+                if verlabel in verprops:
+                    libver = verprops[verlabel]
+                    pname = libname + ' ' + libver + " source:"+file_path
+                else:
+                    pname = libname + " source:"+file_path
+            else:
+                pname = libname + " source:"+file_path
+            if pname not in plist:
+                plist.append(pname)
+
+    if len(plist) > 0:
+        if args.include_unused_dependencies == False:
+            logging.debug("Filtering out unused dotnet dependencies. This may take some time...")
+            plist = filter_used_dotnet_dependencies(args, plist, localpath)
+        else:
+            logging.warn("Including unused dependencies")
+            logging.warn("May increase false positives")
+
+    if len(plist) > 0:
+        return plist, plist
+
+    # Fallback to packages.config file if no dependencies found from .csproj files
+    files = lib_utils.find_files(localpath, 'packages.config')
+    for file_path in files:
+        fp = lib_utils.tw_open(file_path, args.encoding)
+        if fp == None:
+            continue
+        contents = fp.read()
+        xmldoc = None
+        try:
+            xmldoc = minidom.parseString(contents)
+        except Exception:
+            logging.error("Unable to parse package config: %s", file_path)
+            continue
         temp_plist = xmldoc.getElementsByTagName('package')
         for p in temp_plist:
             libname = p.getAttribute('id')
@@ -288,7 +479,16 @@ def discover_packages_config(args, localpath):
             pname = libname + ' ' + libver + " source:"+file_path
             if pname not in plist:
                 plist.append(pname)
-    return plist, None
+
+    if len(plist) > 0:
+        if args.include_unused_dependencies == False:
+            logging.debug("Filtering out unused nuget dependencies. This may take some time...")
+            plist = filter_used_dotnet_dependencies(args, plist, localpath)
+        else:
+            logging.warn("Including unused dependencies")
+            logging.warn("May increase false positives")
+
+    return plist, plist
 
 def discover_yarn(args, localpath):
     plist = []
@@ -296,11 +496,11 @@ def discover_yarn(args, localpath):
     files = lib_utils.find_files(localpath, 'yarn.lock')
     if len(files) == 0 and args.type is not None:
         files = lib_utils.find_files(localpath, 'package.json')
-        plist = process_package_json_files(files, args.level, localpath)
+        plist = process_package_json_files(files, args, localpath)
         return plist
 
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -319,7 +519,8 @@ def discover_yarn(args, localpath):
                 libver = vline.split()[1].replace('"','')
                 if localpath.startswith('/tmp/'):
                     file_path = file_path.replace(localpath+'/','')
-                pname = libname+' '+libver + " source:"+file_path
+                pname = libname+' '+libver
+                pname = cleanse_semver_version(pname) + " source:"+file_path
                 if pname not in plist:
                     plist.append(pname)
                     p1list.append(pname)
@@ -345,7 +546,7 @@ def discover_ruby(args, localpath):
     if len(files) == 0:
         files = lib_utils.find_files(localpath, 'Gemfile.lock')
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         contents = fp.read()
@@ -393,7 +594,7 @@ def discover_python(args, localpath):
     plist = []
     files = lib_utils.find_files(localpath, 'requirements.txt')
     for file_path in files:
-        fp = open(file_path, 'r')
+        fp = lib_utils.tw_open(file_path, args.encoding)
         if fp == None:
             continue
         req = requirements.parse(fp)
@@ -451,18 +652,22 @@ def discover_jar(args, localpath):
         prod = ''
         ver = ''
         #print "Checking "+file_path
-        zf = zipfile.ZipFile(file_path, 'r')
         try:
-            metafile = zf.read('META-INF/MANIFEST.MF')
-            if metafile:
-                for l in metafile.splitlines():
-                    if l.startswith(b"Bundle-Version:"):
-                        ver = l.split(b':')[1].strip()
-                    if l.startswith(b"Bundle-Name:"):
-                        prod = l.split(b':')[1].strip().lower().replace(b' ',b'-')
-        except KeyError:
-            #print "Error: No manifest found"
-            pass
+            zf = zipfile.ZipFile(file_path, 'r')
+            try:
+                metafile = zf.read('META-INF/MANIFEST.MF')
+                if metafile:
+                    for l in metafile.splitlines():
+                        if l.startswith(b"Bundle-Version:"):
+                            ver = l.split(b':')[1].strip()
+                        if l.startswith(b"Bundle-Name:"):
+                            prod = l.split(b':')[1].strip().lower().replace(b' ',b'-')
+            except KeyError:
+                #print "Error: No manifest found"
+                pass
+        except zipfile.BadZipfile:
+            logging.warn("Unable to inspect file: %s", os.path.basename(file_path))
+
         if prod == '' or ver == '':
             jfile = os.path.basename(file_path)
             jfile = jfile.replace('.jar','')
@@ -477,6 +682,10 @@ def discover_jar(args, localpath):
         if localpath.startswith('/tmp/'):
             file_path = file_path.replace(localpath+'/','')
         prod = prod.strip() + " source:"+file_path
+        prod = prod.replace("b'","")
+        prod = prod.replace("'","")
+        prod = prod.replace('b"','')
+        prod = prod.replace('"','')
         if prod != '':
             plist.append(prod)
     return plist, None
@@ -487,6 +696,7 @@ def discover_specified_type(repo_type, args, localpath):
         return [], None
 
     plist = []
+    p1list = []
     if repo_type == 'pip':
         plist, p1list = discover_python(args, localpath)
     elif repo_type == 'ruby':
@@ -507,6 +717,8 @@ def discover_specified_type(repo_type, args, localpath):
         plist, p1list = discover_jar(args, localpath)
     elif repo_type == 'cargo':
         plist, p1list = discover_cargo_toml(args, localpath)
+    elif repo_type == 'go':
+        plist, p1list = discover_go_mod(args, localpath)
 
     return plist, p1list
 
@@ -526,21 +738,31 @@ def strip_source(plist):
             ret_list.append(pname)
     return ret_list
 
-def discover_inventory(args, localpath):
-    default_id_name = get_last_component(args.repo)
+def get_asset_id(args):
     asset_id = None
-    if args.assetid == None:
-        asset_id = default_id_name
+    if args.assetid == None or args.assetid.strip() == "":
+        tokens = [args.handle.split('@')[0]]
+        repo_path = args.repo
+        if repo_path.startswith('http:') or repo_path.startswith('https:'):
+            tokens.extend(repo_path.split('/')[3:])
+            if args.branch is not None and args.branch.strip() != "":
+                tokens.append(args.branch)
+        else:
+            tokens.append(os.path.basename(os.path.normpath(repo_path)))
+        asset_id = "-".join(tokens)
     else:
         asset_id = args.assetid
-    asset_name = None
-    if args.assetname == None:
-        asset_name = default_id_name
-    else:
-        asset_name = args.assetname
     asset_id = asset_id.replace(' ','-')
     asset_id = asset_id.replace('/','-')
     asset_id = asset_id.replace(':','-')
+    return asset_id
+
+def discover_inventory(args, localpath):
+    asset_name = None
+    if args.assetname == None or args.assetname.strip() == "":
+        asset_name = get_last_component(args.repo)
+    else:
+        asset_name = args.assetname
 
     atype = 'Source Repository'
     plist = []
@@ -568,10 +790,10 @@ def discover_inventory(args, localpath):
         asset_tags.append(args.type)
 
     if plist == None or len(plist) == 0:
-        logging.warning("Unable to identify any source code components")
+        logging.info("Unable to identify any source code dependencies")
 
     asset_data = {}
-    asset_data['id'] = asset_id
+    asset_data['id'] = get_asset_id(args)
     asset_data['name'] = asset_name
     asset_data['type'] = atype
     asset_data['owner'] = args.handle
@@ -587,18 +809,24 @@ def on_rm_error( func, path, exc_info):
     os.chmod( path, stat.S_IWRITE )
     os.unlink( path )
 
-def get_inventory(args):
+# note this only handles actual repos and not orgs (i.e. all repos in an org)
+def get_inventory_helper(args):
     path = None
     if args.repo.startswith('http'):
         if os.path.isfile(GIT_PATH) == False:
             logging.error("git executable does not exist at [%s]", GIT_PATH)
             return None
+        dev_null_device = open(os.devnull, "w")
         path = tempfile.mkdtemp()
         base_path = path
         new_repo = None
+        logging.info("Cloning repo locally...")
         try:
-            cmdarr = [GIT_PATH, 'clone', args.repo, path+'/.']
-            out = subprocess.check_output(cmdarr)
+            if args.branch and args.branch != '':
+                cmdarr = [GIT_PATH, 'clone', '--branch', args.branch, args.repo, path+'/.']
+            else:
+                cmdarr = [GIT_PATH, 'clone', args.repo, path+'/.']
+            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
         except:
             logging.error(traceback.format_exc())
             logging.error('Error cloning repo locally')
@@ -619,15 +847,65 @@ def get_inventory(args):
         secret_records = lib_code_secrets.scan_for_secrets(args, path, base_path)
         assets[0]['secrets'] = secret_records
 
+    code_issues = []
     if args.sast:
         logging.info("Performing static analysis. This may take some time.")
         sast_records = sast.run_sast(args, path, base_path)
-        assets[0]['sast'] = sast_records
+        code_issues.extend(sast_records)
+
+    if args.iac_checks:
+        logging.info("Identifying infrastructure as code (IaC) issues. This may take some time.")
+        iac_records = iac.run_iac_checks(args, path, base_path)
+        code_issues.extend(iac_records)
+
+    if len(code_issues) > 0:
+        assets[0]['sast'] = code_issues
 
     if args.repo.startswith('http'):
         shutil.rmtree(path, onerror = on_rm_error)
 
-    if len(assets[0]['products']) == 0 and (assets[0].get('secrets')!= None and len(assets[0]['secrets']) == 0):
-        logging.warning("No products idenitified nor any code secrets found.")
-        return [] # if there are no products nor secrets then no assets to report
+    if len(assets[0]['products']) == 0 and (assets[0].get('secrets') is None or len(assets[0]['secrets']) == 0) and (assets[0].get('sast') is None or len(assets[0]['sast']) == 0):
+        logging.warning("Nothing to report")
+        if args.create_empty_asset is None or not args.create_empty_asset:
+            return [] # if there are no products nor secrets then no assets to report
     return assets
+
+def get_inventory(args):
+    if args.gh_user:
+        owner = args.gh_user
+        dev_null_device = open(os.devnull, "w")
+        # check if 'gh' command is available
+        try:
+            cmdarr = ['which', 'gh']
+            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
+        except subprocess.CalledProcessError as e:
+            logging.error("[gh] command not found")
+            return None
+        # check if user is logged in 'gh'
+        try:
+            cmdarr = ['gh', 'auth', 'status']
+            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
+        except subprocess.CalledProcessError as e:
+            logging.error("Please login using [gh auth login] command to list repositories")
+            logging.debug("[gh auth status] command returned exit code [%s]", e.returncode)
+            return None
+        # list repo for org/user
+        try:
+            cmdarr = ['gh', 'repo', 'list', owner, '--json', 'name,url', '--limit', '65535']
+            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
+        except subprocess.CalledProcessError as e:
+            logging.error("Unable to list repos for [%s]", owner)
+            logging.debug("[gh repo list %s --json name,url] command returned %s", owner, e.output)
+            logging.debug("[gh repo list %s --json name,url] command returned exit code [%s]", owner, e.returncode)
+            return None
+        repos = json.loads(out.strip())
+        assets = []
+        for repo in repos:
+            logging.info("Discovering repo [%s] as an asset", repo['name'])
+            args.repo = repo['url']
+            logging.info("Repo URL: %s", args.repo)
+            temp_assets = get_inventory_helper(args)
+            assets.extend(temp_assets)
+        return assets
+    else:
+        return get_inventory_helper(args)
